@@ -171,7 +171,7 @@ int main(int argc, char **argv) {
       if (!check_ptr->loadDataForDate(mydate))
       {
 	 std::time_t x = check_ptr->closestValidData(mydate);
-	 std::cerr << PACKAGE ": Warning: The available bankdata is not valid today; closest available date with valid bankdata is: " << ctime(&x)
+	 std::cerr << PACKAGE ": Warning: The bankdata files which are known to your installed version of libktoblzcheck are not valid today.  If you think up-to-date files should be available, you will have to update the installed libktoblzcheck library as well.  The closest known available date with valid bankdata is: " << ctime(&x)
 		   << std::endl;
       }
    }
