@@ -44,9 +44,9 @@
 // to function is done by the cb_funcs[] array down at the end of this
 // file. At runtime, the cb_funcs[] array will be inserted into a hash
 // table, so that the lookup of each method's function will be
-// *really* fast. 
+// *really* fast.
 //
-// When adding a new method, do this: 
+// When adding a new method, do this:
 //
 // 1. Write a method_XY() function similar to the ones existing here.
 //
@@ -58,7 +58,7 @@
 
 AccountNumberCheck::Result method_00(const int *account, int *weight) {
     number2Array("2121212120", weight);
-    return algo01(10, weight, true, 10, account);	
+    return algo01(10, weight, true, 10, account);
 }
 AccountNumberCheck::Result method_01(const int *account, int *weight) {
     number2Array("1731731730", weight);
@@ -70,7 +70,7 @@ AccountNumberCheck::Result method_02(const int *account, int *weight) {
 }
 AccountNumberCheck::Result method_03(const int *account, int *weight) {
     number2Array("2121212120", weight);
-    return algo01(10, weight, false, 10, account);	
+    return algo01(10, weight, false, 10, account);
 }
 // this is method "02" using modified weight
 AccountNumberCheck::Result method_04(const int *account, int *weight) {
@@ -116,14 +116,14 @@ AccountNumberCheck::Result method_11(const int *account, int *weight) {
     if (10 == tmp)  //use 9 instead of 0
 	tmp = 9;
     else
-	tmp = tmp % 10; 
+	tmp = tmp % 10;
     //tmp = (11 - tmp) % 10;
     //if (10 == tmp)
     // tmp = 9;
 
     if  (tmp == account[9])
 	return AccountNumberCheck::OK;
-    else 
+    else
 	return AccountNumberCheck::ERROR;
 }
 AccountNumberCheck::Result method_12(const int *account, int *weight) {
@@ -138,7 +138,7 @@ AccountNumberCheck::Result method_13(const int *account, int *weight) {
 	number2Array(array2Number(account).substr(2) + "00", account_buf);
 	if (AccountNumberCheck::OK != algo01(10, weight, true, 8, account_buf))
 	    return AccountNumberCheck::ERROR;
-    } 
+    }
     return AccountNumberCheck::OK;
 }
 // this is method "02" using modified weight
@@ -203,7 +203,7 @@ AccountNumberCheck::Result method_21(const int *account, int *weight) {
 	int tmp = result % 10;
 	result /= 10;
 	result += tmp;
-    }	
+    }
     result = 10 - result;
 
     if (result == account[9])
@@ -224,7 +224,7 @@ AccountNumberCheck::Result method_23(const int *account, int *weight) {
 	return AccountNumberCheck::OK;
     if (1 < tmp && account[6] == (11 - tmp))
 	return AccountNumberCheck::OK;
-	
+
     return AccountNumberCheck::ERROR;
 }
 // rewrite of method_24(), moved part of the code from algorithms.c old algo05() here
@@ -247,7 +247,7 @@ AccountNumberCheck::Result method_24(const int *account, int *weight) {
     }
 
     // check how many leading "0", shift weight to right
-    std::string weightString = "";  
+    std::string weightString = "";
     for (int i=0; i<10; i++) {
 	if (0 == account_buf[i])
 	    weightString += "0";
@@ -293,7 +293,7 @@ AccountNumberCheck::Result method_26(const int *account, int *weight) {
 }
 // Added by Jens Gecius, validated with nine own accountIDs + testaccountID (six > 1000000000)
 AccountNumberCheck::Result method_27(const int *account, int *weight) {
-    int transform[6][10] = 
+    int transform[6][10] =
 	{ {0,0,0,0,0,0,0,0,0,0}, // zero for non-transform [0][x]
 	  {0,1,5,9,3,7,4,8,2,6}, // first transform line   [1][x]
 	  {0,1,7,6,9,8,3,2,5,4}, // second transform line  [2][x]
@@ -316,7 +316,7 @@ AccountNumberCheck::Result method_28(const int *account, int *weight) {
 }
 // Added by Jens Gecius, validated with test accountID
 AccountNumberCheck::Result method_29(const int *account, int *weight) {
-    int transform[6][10] = 
+    int transform[6][10] =
 	{ {0,0,0,0,0,0,0,0,0,0}, // zero for non-transform [0][x]
 	  {0,1,5,9,3,7,4,8,2,6}, // first transform line   [1][x]
 	  {0,1,7,6,9,8,3,2,5,4}, // second transform line  [2][x]
@@ -396,11 +396,11 @@ AccountNumberCheck::Result method_40(const int *account, int *weight) {
 AccountNumberCheck::Result method_41(const int *account, int *weight) {
     number2Array("2121212120", weight);
     if (9 == account[3]) {
-	weight[0] = 0; 
+	weight[0] = 0;
 	weight[1] = 0;
 	weight[2] = 0;
     }
-    return algo01(10, weight, true, 10, account);	
+    return algo01(10, weight, true, 10, account);
 }
 AccountNumberCheck::Result method_42(const int *account, int *weight) {
     number2Array("0987654320", weight);
@@ -421,7 +421,7 @@ AccountNumberCheck::Result method_45(const int *account, int *weight) {
 	return AccountNumberCheck::OK;
 
     number2Array("2121212120", weight);
-    return algo01(10, weight, true, 10, account);	
+    return algo01(10, weight, true, 10, account);
 }
 AccountNumberCheck::Result method_46(const int *account, int *weight) {
     number2Array("0065432000", weight);
@@ -485,7 +485,7 @@ AccountNumberCheck::Result method_51(const int *account, int *weight) {
 	return AccountNumberCheck::ERROR;  // Invalid IDs
     return algo01(7, weight, false, 10, account); // Method C
 }
-AccountNumberCheck::Result method_52(const int *account, int *weight, 
+AccountNumberCheck::Result method_52(const int *account, int *weight,
 				     const std::string& accountId, const std::string& bankId) {
     // Changed by Jens Gecius, only 10 digit accountIDs starting with "9",
     // *NOT* validated
@@ -554,7 +554,7 @@ AccountNumberCheck::Result method_57(const int *account, int *weight) {
 	if ((777777 == firstSix) || (888888 == firstSix))	// Exception for variant 1
 	  return AccountNumberCheck::OK;
 	number2Array("1212121210", weight);
-	return algo01(10, weight, true, 10, account);	
+	return algo01(10, weight, true, 10, account);
     } else if (	( firstTwo >= 32 && firstTwo <= 39 ) ||
 	( firstTwo >= 41 && firstTwo <= 49 ) ||
 	( firstTwo >= 52 && firstTwo <= 54 ) ||
@@ -562,7 +562,7 @@ AccountNumberCheck::Result method_57(const int *account, int *weight) {
 	( firstTwo >= 67 && firstTwo <= 69 ) ||
 	( firstTwo >= 83 && firstTwo <= 87 ) ||
 	( firstTwo >= 96 && firstTwo <= 98 ) ||
-	firstTwo == 62 || firstTwo == 63 || firstTwo == 71 || 
+	firstTwo == 62 || firstTwo == 63 || firstTwo == 71 ||
 	firstTwo == 72 || firstTwo == 74 || firstTwo == 89 ||
 	firstTwo == 90 || firstTwo == 92 || firstTwo == 93 ) {	// Variante 2
 	number2Array("1201212121", weight);
@@ -657,7 +657,7 @@ AccountNumberCheck::Result method_66(const int *account, int *weight) {
 }
 AccountNumberCheck::Result method_67(const int *account, int *weight) {
     number2Array("2121212000", weight);
-    return algo01(10, weight, true, 8, account);	
+    return algo01(10, weight, true, 8, account);
 }
 AccountNumberCheck::Result method_68(const int *account, int *weight) {
    // FIXME: This needs to be checked:
@@ -693,7 +693,7 @@ AccountNumberCheck::Result method_68(const int *account, int *weight) {
 }
 // Added by Jens Gecius, validated with three test accountIDs
 AccountNumberCheck::Result method_69(const int *account, int *weight) {
-    int transform[6][10] = 
+    int transform[6][10] =
 	{ {0,0,0,0,0,0,0,0,0,0}, // zero for non-transform [0][x]
 	  {0,1,5,9,3,7,4,8,2,6}, // first transform line   [1][x]
 	  {0,1,7,6,9,8,3,2,5,4}, // second transform line  [2][x]
@@ -722,7 +722,7 @@ AccountNumberCheck::Result method_70(const int *account, int *weight) {
     return algo01(11, weight, false, 10, account);
 }
 AccountNumberCheck::Result method_71(const int *account, int *weight) {
-    number2Array("0654321000", weight);	
+    number2Array("0654321000", weight);
     int tmp = algo03(11, weight, false, account, 0, 9);
 
     // no rest after division by modulus (in algo03)?
@@ -730,7 +730,7 @@ AccountNumberCheck::Result method_71(const int *account, int *weight) {
 	// Changed by Jens Gecius, validated with one test accountID
 	// this way we get 0 as final result
 	tmp = 11;
-    // if 1 rest, keep it! 
+    // if 1 rest, keep it!
     if (1 == tmp)
 	tmp = 10;
 
@@ -819,9 +819,9 @@ AccountNumberCheck::Result method_77(const int *account, int *weight) {
 AccountNumberCheck::Result method_78(const int *account, int *weight) {
     if (0 == account[0] && 0 == account[1])
 	return AccountNumberCheck::OK;
-	
+
     number2Array("2121212120", weight);
-    return algo01(10, weight, true, 10, account);	
+    return algo01(10, weight, true, 10, account);
 }
 AccountNumberCheck::Result method_79(const int *account, int *weight) {
     // Modified by Jens Gecius, not verified as no bank seems to use it...
@@ -890,7 +890,7 @@ AccountNumberCheck::Result method_83(const int *account, int *weight) {
 	return AccountNumberCheck::ERROR;
 
     number2Array("0087654320", weight);
-    return algo01(11, weight, false, 10, account);	
+    return algo01(11, weight, false, 10, account);
 }
 AccountNumberCheck::Result method_84(const int *account, int *weight) {
     // exception "Ausnahme f. Sachkonten"
@@ -987,7 +987,7 @@ AccountNumberCheck::Result method_89(const int *account, int *weight) {
 }
 AccountNumberCheck::Result method_90(const int *account, int *weight) {
     // variant a
-    number2Array("0007654320", weight);	
+    number2Array("0007654320", weight);
     if (AccountNumberCheck::OK == algo01(11, weight, false, 10, account))
 	return AccountNumberCheck::OK;
 
@@ -995,11 +995,11 @@ AccountNumberCheck::Result method_90(const int *account, int *weight) {
     number2Array("0000654320", weight);
     if (AccountNumberCheck::OK == algo01(11, weight, false, 10, account))
 	return AccountNumberCheck::OK;
-	
+
     // variant c
     if (AccountNumberCheck::OK == algo01(07, weight, false, 10, account))
 	return AccountNumberCheck::OK;
-	
+
     // variant d
     if (AccountNumberCheck::OK == algo01(9, weight, false, 10, account))
 	return AccountNumberCheck::OK;
@@ -1018,13 +1018,13 @@ AccountNumberCheck::Result method_91(const int *account, int *weight) {
     AccountNumberCheck::Result tmp = algo01(11, weight, false, 7, account);
     if (AccountNumberCheck::OK == tmp)
 	return AccountNumberCheck::OK;
-    else {	
+    else {
 	number2Array("2345670000", weight);
 	tmp = algo01(11, weight, false, 7, account);
 	if (AccountNumberCheck::OK == tmp)
 	    return AccountNumberCheck::OK;
-	else {	
-	    number2Array("0987650432", weight); weight[0] = 10;	
+	else {
+	    number2Array("0987650432", weight); weight[0] = 10;
 	    tmp = algo01(11, weight, false, 7, account);
 	    if (AccountNumberCheck::OK == tmp)
 		return AccountNumberCheck::OK;
@@ -1046,7 +1046,7 @@ AccountNumberCheck::Result method_93(const int *account, int *weight) {
     number2Array("6543200000", weight);
     if ("0000" == array2Number(account).substr(0, 4)) {
 	checkIndex = 10;
-	number2Array("0000654320", weight);	  
+	number2Array("0000654320", weight);
     }
 
     if (AccountNumberCheck::OK == algo01(11, weight, false, checkIndex, account))
@@ -1094,10 +1094,10 @@ AccountNumberCheck::Result method_97(const int *account, int *weight) {
     long_long valueX = number2LongLong(tmp);
     valueX -= (valueX / 11) * 11;
     valueX = valueX % 10;
-	
+
     if (valueX == account[9])
 	return AccountNumberCheck::OK;
-    else 
+    else
 	return AccountNumberCheck::ERROR;
 }
 AccountNumberCheck::Result method_98(const int *account, int *weight) {
@@ -1111,7 +1111,7 @@ AccountNumberCheck::Result method_98(const int *account, int *weight) {
 }
 AccountNumberCheck::Result method_99(const int *account, int *weight) {
     // some numbers do not get checked
-    if ("0396000000" <= array2Number(account) && 
+    if ("0396000000" <= array2Number(account) &&
 	array2Number(account) <= "0499999999")
 	return AccountNumberCheck::OK;
     number2Array("4327654320", weight);
@@ -1183,7 +1183,7 @@ AccountNumberCheck::Result method_A4(const int *account, int *weight) {
     number2Array("6543200000", weight);
     if ("0000" == array2Number(account).substr(0, 4)) {
 	checkIndex = 10;
-	number2Array("0000654320", weight);	  
+	number2Array("0000654320", weight);
     }
 
     if (AccountNumberCheck::OK == algo01(11, weight, false, checkIndex, account))
@@ -1323,12 +1323,19 @@ AccountNumberCheck::Result method_B7(const int *account, int *weight) {
     else
 	return AccountNumberCheck::OK;		// variant 2, "method 9" :-))
 }
-// new method as of September 5th, 2005, checked with Bundesbank-Testnumbers
+// method B8 was last changed for due-date June, 6 2011
 AccountNumberCheck::Result method_B8(const int *account, int *weight) {
     if (AccountNumberCheck::OK == method_20(account,weight) ) // variant 1
 	return AccountNumberCheck::OK;
-    else
-	return method_29(account,weight);	// variant 2
+    if (AccountNumberCheck::OK == method_29(account,weight) ) // variant 2
+	return AccountNumberCheck::OK;
+    if (account[0] == 5 && account[1] > 0 && account[1] <= 9) // variant 3
+	return AccountNumberCheck::OK;
+    if (account[0] == 9 &&
+	 ((account[1] == 0 && account[2] > 0 && account[2] <= 9) ||
+	  (account[1] == 1 && account[2] == 0)))
+	return AccountNumberCheck::OK;
+    return AccountNumberCheck::ERROR;
 }
 // B9 and C0 added by Alexander Kurz
 // new method as of December 5th, 2005, checked with Bundesbank-Testnumbers
@@ -1396,7 +1403,7 @@ AccountNumberCheck::Result method_C4(const int *account, int *weight) {
     }
 }
 
-// new methods C5 and C6 valid from September 3rd, 2007, checked with Bundesbank-Testnumbers
+// new method C5 valid from September 3rd, 2007, checked with Bundesbank-Testnumbers
 AccountNumberCheck::Result method_C5(const int *account, int *weight) {
     int len	= 10;
     while(account[10-len]==0 && len>0)	 { len--; }
@@ -1425,7 +1432,7 @@ AccountNumberCheck::Result method_C5(const int *account, int *weight) {
     }
     return AccountNumberCheck::ERROR;
 }
-// method C6 was last changed for due-date June, 6 2010
+// method C6 was last changed for due-date June, 6 2011
 // checked with Bundesbank-Testnumbers
 AccountNumberCheck::Result method_C6(const int *account, int *weight) {
     number2Array("0121212120", weight);
@@ -1437,6 +1444,8 @@ AccountNumberCheck::Result method_C6(const int *account, int *weight) {
 	case 1:	mod_factor=33;	break;	// prefix="4451981"
 	case 2:	mod_factor=36;	break;	// prefix="4451992"
 	case 3:	mod_factor=38;	break;	// prefix="4451993"
+	case 5:	mod_factor=41;	break;	// prefix="4344990"
+	case 6:	mod_factor=43;	break;	// prefix="4344991"
 	case 7:	mod_factor=31;	break;	// prefix="5499570"
 	case 9:	mod_factor=40;	break;	// prefix="5499579"
 	default:
@@ -1449,7 +1458,7 @@ AccountNumberCheck::Result method_C6(const int *account, int *weight) {
     // compare the result with the real check number
     if (account[9] == result)
 	return AccountNumberCheck::OK;
-    else 
+    else
 	return AccountNumberCheck::ERROR;
 }
 // new method as of December 3rd, 2007, checked with Bundesbank-Testnumbers
@@ -1482,22 +1491,22 @@ AccountNumberCheck::Result method_D0(const int *account, int *weight) {
 	return AccountNumberCheck::OK;				// variant 2, "method 9" again :-))
 }
 // method D1 was modified with due date starting from 7. June 2010
+// method D1 was last modified for due date 7. March 2011
 AccountNumberCheck::Result method_D1(const int *account, int *weight) {
+    if (account[0] == 2 || account[0] == 7 || account[0] == 8)
+	return AccountNumberCheck::ERROR;
     // note: this method is quite similar to method_C6
     number2Array("2121212120", weight);
     int result	= algo03(10, weight, true, account, 0, 8);
-    // quick hack: the processed number-prefix "436338" or "428259" boils
+    // quick hack: the processed number-prefix "436338" boils
     //		   down to a constant factor
-    if (account[0] == 0 || ( account[0] >= 3 && account[0] <= 5 ) || account[0] == 9)
-	result	+= 31;	// prefix "436338" -> factor 31
-    else
-	result	+= 29;	// prefix "428259" -> factor 29
+    result	+= 31;	// prefix "436338" -> factor 31
     // calc the check number, note: 1000 should be large enough.
     result	= (1000 - result) % 10;
     // compare the result with the real check number
     if (account[9] == result)
 	return AccountNumberCheck::OK;
-    else 
+    else
 	return AccountNumberCheck::ERROR;
 }
 
@@ -1515,10 +1524,11 @@ AccountNumberCheck::Result method_D3(const int *account, int *weight) {
     return method_27(account,weight);				// variant 2
 }
 
-// new method D4 valid from June 7th, 2010, checked with Bundesbank-Testnumbers
+// method D4 was last changed for due-date June, 6 2011
+// checked with Bundesbank-Testnumbers
 AccountNumberCheck::Result method_D4(const int *account, int *weight) {
     // note this is a slightly modified version of the old method D1
-    if (( account[0] >= 0 && account[0] <= 2 ) || ( account[0] >= 6 && account[0] <= 8 ))
+    if (account[0] == 0)
 	return AccountNumberCheck::ERROR;
     // note: from here on this method is quite similar to method_C6
     number2Array("2121212120", weight);
@@ -1553,6 +1563,36 @@ AccountNumberCheck::Result method_D5(const int *account, int *weight) {
     }
 }
 
+// new method D6 valid from March 6th, 2011, checked with Bundesbank-Testnumbers
+AccountNumberCheck::Result method_D6(const int *account, int *weight) {
+    if (AccountNumberCheck::OK == method_07(account,weight) )	// variant 1
+	return AccountNumberCheck::OK;
+    if (AccountNumberCheck::OK == method_03(account,weight) )	// variant 2
+	return AccountNumberCheck::OK;
+    return method_00(account,weight);	// variant 3
+}
+
+// new method D7 valid from March 6th, 2011, checked with Bundesbank-Testnumbers
+AccountNumberCheck::Result method_D7(const int *account, int *weight) {
+    number2Array("2121212120", weight);
+    int tmp = algo03(10, weight, true, account, 0, 9);
+    if  (tmp == account[9])
+	return AccountNumberCheck::OK;
+    else
+	return AccountNumberCheck::ERROR;
+}
+
+// new method D8 valid from March 6th, 2011, checked with Bundesbank-Testnumbers
+AccountNumberCheck::Result method_D8(const int *account, int *weight) {
+    number2Array("2121212120", weight);
+    if (account[0] > 0 && account[0] <= 9)	// variant 1
+	return algo01(10, weight, true, 10, account);
+    if (account[0] == 0 && account[1] == 0 &&
+	account[2] > 0 && account[2] <= 9)	// variant 2
+	return AccountNumberCheck::OK;		// "method09"
+    return AccountNumberCheck::ERROR;
+}
+
 struct method_func_s {
     const char *str;
     AccountNumberCheck::MethodFunc func;
@@ -1690,6 +1730,9 @@ const struct method_func_s cb_funcs[] = {
   { "D3", method_D3},
   { "D4", method_D4},
   { "D5", method_D5},
+  { "D6", method_D6},
+  { "D7", method_D7},
+  { "D8", method_D8},
   { 0, 0} // Important: The array has to end with the {0,0} entry,
 	  // otherwise initMethodMap() will runaway and crash!
 };
